/*
 * Copyright (c) 1999 Venkat. All Rights Reserved.
 *
 * The author (Venkat) grants everyone a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and 
 * binary code form, provided that: 
 * i) this copyright notice appear on all copies of the software; and 
 * ii) the software is not utilized in a manner which is disparaging 
 * to the author.
 */

package com.venkat.RMIDemo;

import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.LocateRegistry;

public
class 
	BBServer
extends
	UnicastRemoteObject
implements
	ScoreSender, Runnable
{
	private Thread theThread = null;
    private Vector theReceiverList;
	private CurrentScore theCurrentScore;

    public BBServer() throws RemoteException 
    {
		super();
		theReceiverList = new Vector();
		theCurrentScore = new CurrentScore();
	}

    public synchronized void register(ScoreReceiver aRemoteReceiver)
	{
		System.out.println("BBServer registration");
		
		theReceiverList.addElement( aRemoteReceiver );

		int winningTotal = theCurrentScore.getWinningTotal();
		int losingTotal = theCurrentScore.getLosingTotal();
		
		try 
		{
			aRemoteReceiver.bringUpToDate( winningTotal, losingTotal );
		}
		catch (RemoteException e)
		{
			e.printStackTrace();
		}

		if (theThread == null)
		{
			theThread = new Thread(this, "RemoteAppletCallback");
			theThread.start();
		}
	}

    public void cancel(ScoreReceiver aRemoteReceiver)
    {
		boolean removed;
		removed = theReceiverList.removeElement( aRemoteReceiver );
		if( removed == true )
		{
			System.out.println("Removed the Receiver");
		}
		else
		{
			System.out.println("Couldn't Remove the Receiver");
		}
		if( theReceiverList.isEmpty() == true )
		{
			// in case current thread is theThread
			theThread = null;
			System.out.println("All Registered Receivers were removed");
			System.out.println("Stopping BBServer");
			System.exit(0);
		}
    }

    public void run() 
    {

		ShotAttempt aShotAttempt = null; 
		BBEventGenerator aGenerator = null;
		ScoreReceiver aReceiver = null;

		while (true)
		{
			try
			{
				// wait for a few seconds between updates
				// GENERATE A BB EVENT, namely a SHOT ATTEMPT
				// pass the shot attempt to updatemethod
				
				aGenerator = new BBEventGenerator();
				aShotAttempt = aGenerator.generateBBEvent();
				
				Thread.currentThread().sleep(5000);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			
			try
			{
	    		System.out.println("BBServer issuing an update");
				for (Enumeration e = theReceiverList.elements(); e.hasMoreElements(); )
				{
					aReceiver = ( (ScoreReceiver) (e.nextElement()) );
					aReceiver.update( aShotAttempt );
				}
				//synchronized (this)
				{
					theCurrentScore.update( aShotAttempt ); 
				}
			}
			catch (RemoteException e)
			{
				e.printStackTrace();
				System.out.println("Remote Client terminated");
				System.out.println("Remote Server stopping updates");
				boolean removed;
				removed = theReceiverList.removeElement( aReceiver );
				aReceiver = null;
				if( theReceiverList.isEmpty() == true )
				{
					System.out.println("All Registered Receivers were removed");
					System.out.println("Stopping BBServer");
					theThread = null;
					System.exit(0);
				}
			}
		}
	}
	

    public static void main(String args[]) 
    {
	
		System.setSecurityManager(new RMISecurityManager());

		try
		{
			LocateRegistry.createRegistry(1099);
			BBServer aServer = new BBServer();
			Naming.rebind("//:1099/com.venkat.RMIDemo.BBServer", aServer);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
    }

} // end BBServer class
